use std::borrow::Cow;

use anyhow::anyhow;
use toml_edit::{Document, Entry, Formatted, Item, Value};

pub fn normalize_version(original: &str) -> anyhow::Result<Cow<str>> {
    let mut toml: Document = original.parse()?;

    let Some(package_item) = toml.get_mut("package") else {
        return Err(anyhow!("Cargo.toml invalid: no [package] table"));
    };

    let Some(package) = package_item.as_table_mut() else {
        return Err(anyhow!("Cargo.toml invalid: [target] is not a table"));
    };

    let Entry::Occupied(mut version_entry) = package.entry("version") else {
        return Err(anyhow!("Cargo.toml invalid: package.version is not set"));
    };

    let Some(version_value) = version_entry.get().as_value() else {
        return Err(anyhow!("Cargo.toml invalid: package.version is not a value"));
    };

    let Some(version) = version_value.as_str() else {
        return Err(anyhow!("Cargo.toml invalid: package.version is not a string"));
    };

    let replacement = if let Some((version, _meta)) = version.split_once('+') {
        String::from(version)
    } else {
        return Ok(Cow::Borrowed(original));
    };

    version_entry.insert(Item::Value(Value::String(Formatted::new(replacement))));

    Ok(Cow::Owned(toml.to_string()))
}
